
 /* House_code_gen.c        Metaprat       6th April 1992  */


inherit "/players/metaprat/houses/house_toolkit_obj";

#include "house_defines.h"


/* standard functions */


id(str) {
   return (str == "house code generator") ||
          (str == "code generator")       ||
          (str == "generator");
}


short() {
   return "house code generator";
}


long(str) {
   write("This object allows the wizard owner to generate LPC code that\n");
   write("is a reasonable copy of the room.  Exits will be all of the\n");
   write("'normal' type, and there will not be any 'furnish' capabilities\n");
   write("in the generated code.\n");
   write("The syntax is:    generate_lpc <full file name>\n");
   write("The code that is generated will be in the filename specified.\n");
   write("eg)    generate_lpc /players/metaprat/rooms/house_7.c\n");
   write("The best thing to call the eventual file is 'house_' followed\n");
   write("by the index number of the room (its on the map).\n");
   write("If you don't specify a filename, it'll generate into /rooms/,\n");
   write("in your directory, assuming you have a directory called that.\n");
}



toolkit_init() {
   if (IAMWIZ) {
      add_action("generate_lpc", "generate_lpc");
   }
}


generate_lpc(str) {
   int    counter;
   string this_line;
   string remainder;
   string filenm;
   string termnm;
   string pathnm;
   string discard;
   string discard2;
   string poss_exits;
   int    monlev;
   string monadj1;
   string monadj2;
   string monname;
   string new_name;
   string mondesc;
   string furnish_list;
   string clock_list;

   if (!environment(this_player())->confirm_house()) {
      write("You are not in a house room.\n");
      return 1;
   }
   main_hall       = environment(this_player())->query_main_hall();
   this_room_index = environment(this_player())->query_room_index();
   furnish_list    = main_hall->get_rooms_furnishings(this_room_index);
   if (!furnish_list || !pointerp(furnish_list)) {
      furnish_list = ({ });
   }

   /* Work out the path to the file (pathnm), its terminal name (termnm) */
   /* and the file that is being appended to (filenm).                   */
   if (!str) {
      filenm = "/players/" + MYNAME + "/rooms/house_" + this_room_index + ".c";
      termnm = "house_" + this_room_index + ".c";
      pathnm = "/players/" + MYNAME + "/rooms";
   }
   else {
      filenm = str;
      if ((sscanf(str, "/open/%s", discard) == 1) ||
          (sscanf(str, "/players/" + MYNAME + "/%s", discard) == 1)) {
         filenm = str;
         /* build up the directory that the file will be in */
         pathnm = "";
         while (sscanf(filenm, pathnm + "/%s/%s", discard, discard2) == 2) {
            pathnm = pathnm + "/" + discard;
         }
         sscanf(filenm, pathnm + "/%s", termnm);
         if (termnm != ("house_" + this_room_index + ".c")) {
            write("Beware, if you generate other rooms, they will be\n");
            write("using the name 'house_" + this_room_index + ".c' to " +
                  "refer to this one.\n");
         }
      }
      else {
         write("You can only write into /open, or your own directory.\n");
         write("You have to supply the full path, eg)\n");
         write("   generate_lpc /players/" + MYNAME + "/rooms/room17.c\n");
         write("if you don't supply a pathname, it will try to generate\n");
         write("code into   /players/" + MYNAME + "/rooms/.\n");
         return 1;
      }
   }

   write("Generating LPC code for the current room.\n");
   write("Target: " + filenm + "\n");
   if (!this_player()->valid_write(filenm)) {
      write("ERROR!  You do not have write access to " + filenm + "\n");
      write("generate_lpc failed.\n");
      return 1;
   }

   if (file_size(pathnm) != -2) {
      write("ERROR!  " + pathnm + " is not be a directory.\n");
      write("Check if the directory exists, and if not, make it.\n");
      return 1;
   }
   if (file_size(filenm) != -1) {
      write("ERROR!  " + filenm + " already exists!\n");
      return 1;
   }
   pathnm = "\"" + pathnm + "/\"";
   write_file(filenm, "/* Metaprat - automatically generated house room */\n");
   write_file(filenm, "\n");
   write_file(filenm, "/* Save as: " + pathnm + termnm + " */\n");
   write_file(filenm, "\n");
   write_file(filenm, "inherit \"/room/room.c\";\n");
   write_file(filenm, "\n");
   write_file(filenm, "#define TUNE_MONSTER_WC(WC) ((WC + 1) * 110 / 100)\n");
   write_file(filenm, "#define TUNE_MONSTER_AC(AC) ((AC + 1) * 110 / 100)\n");
   write_file(filenm, "#define TUNE_MONSTER_HP(HP) ((HP + 0) * 130 / 100)\n");
   write_file(filenm, "#define ROOMSDIR  " + pathnm + "\n");
   write_file(filenm, "#define PREFIX    \"house_\"\n");
   write_file(filenm, "\n");
   write_file(filenm, "/* object declarations for monsters etc go here */\n");
   write_file(filenm, "object mon_ob;\n");
   write_file(filenm, "\n");
   write_file(filenm, "reset(arg) {\n");
   write_file(filenm, "   /* This routine is called with arg == 0 when */\n");
   write_file(filenm, "   /* the room is created, and arg == 1 later.  */\n");
   write_file(filenm, "   if (!arg) {\n");
   write_file(filenm, "      no_castle_flag = 1;\n");


   if (search_array(furnish_list, "darkness")) {
      write_file(filenm, "      set_light(0);\n");
   }
   else {
      write_file(filenm, "      set_light(1);\n");
   }
   write_file(filenm, "\n");


   /* code for the directions array */
   write_file(filenm, "      /* Directions are here, and is an array    */\n");
   write_file(filenm, "      /* containing exit destination / exit verb */\n");
   write_file(filenm, "      dest_dir =  ({\n");
   poss_exits = main_hall->get_valid_directions();
   for (counter = 0; counter < sizeof(poss_exits); counter ++) {
      if (main_hall->get_exit_status(this_room_index,
                                     poss_exits[counter]) != DOOR_NONE) {
         if (main_hall->valid_cardinal(poss_exits[counter])) {
            /* Its an exit to another room inside the house */
            write_file(filenm,
                       "         ROOMSDIR + PREFIX + \"" +
                       main_hall->direction_room(this_room_index,
                                                 poss_exits[counter]));
         }
         else {
            /* an exit to a room outside the house */
            write_file(filenm,
                       "         \"" +
                       main_hall->file_name_exit(this_room_index,
                                                 poss_exits[counter]));
         }
         write_file(filenm, "\", \"" + poss_exits[counter] + "\"");
         if (counter < sizeof(poss_exits) - 2) {
           write_file(filenm, ",");
         }
         write_file(filenm, "\n");
      }
   }
   write_file(filenm, "            });\n");


   /* code for the short description */
   write_file(filenm, "      /* Brief room description */\n");
   write_file(filenm, "      short_desc = \"" +
              main_hall->get_rooms_short(this_room_index) + "\";\n");


   /* Now produce the long description code */
   write_file(filenm, "      /* Description of the room*/\n");
   write_file(filenm, "      long_desc =\n");
   discard = main_hall->get_rooms_long(this_room_index);
   while (sscanf(discard, "%s\n%s", this_line, discard2) == 2) {
      if ((discard2) && (discard2 != "")) {
         write_file(filenm, "\"" + this_line + "\\n\" +\n");
      }
      else {
         write_file(filenm, "\"" + this_line + "\\n\";\n");
      }
      discard = discard2;
   }
   discard2 = "";


   /* Now the items array */
   discard = main_hall->get_rooms_items(this_room_index);
   write_file(filenm, "      /* examinable items */\n");
   write_file(filenm, "      items = ({\n");
   for (counter = 0; counter < sizeof(discard) - 1; counter += 2) {
      write_file(filenm, "\"" + discard[counter] + "\", \"" +
                         discard[counter + 1] + "\"");
      if (counter < sizeof(discard) - 3) {
        write_file(filenm, ",");
      }
      write_file(filenm, "\n");
   }
   write_file(filenm, "      });\n");

   write_file(filenm, "      /* Permanent objects should be cloned here */\n");
   write_file(filenm, "      /* since this is only executed once.       */\n");
   write_file(filenm, "   }\n");
   write_file(filenm, "   /* If you have any monsters or movable objs */\n");
   write_file(filenm, "   /* you want to clone into this room, do it  */\n");
   write_file(filenm, "   /* here, since its executed every reset.    */\n");
   for (counter = 0; counter < sizeof(furnish_list); counter ++) {
      if (sscanf(furnish_list[counter], "monster:%d:%s %s %s",
                 monlev, monadj1, monadj2, monname) == 4) {
         break;
      }
      monlev = 0;
   }
   if (monlev && sscanf(monname, "%s:%s", new_name, mondesc) == 2) {
      monname = new_name;
   }
   else {
      mondesc = 0;
   }
   if (monlev) {
      write_file(filenm, "   if (!mon_ob) {\n");
      write_file(filenm, "      mon_ob = clone_object(\"obj/monster\");\n");
      write_file(filenm, "      mon_ob->set_name(\"" +
        monadj1 + " " + monadj2 + " " + monname + "\");\n");
      write_file(filenm, "      mon_ob->set_level(" + monlev + ");\n");
      write_file(filenm, "      /* mon_ob->set_short();          */\n");
      if (!mondesc) {
         write_file(filenm, "      /* mon_ob->set_long();           */\n");
      }
      else {
         write_file(filenm, "      mon_ob->set_long(\"" +
                    capitalize(mondesc) + "\\n\");\n");
      }
      write_file(filenm, "      mon_ob->set_alias(\"" +
        monadj1 + " " + monname + "\");\n");
      write_file(filenm, "      mon_ob->set_alt_name(\"" +
        monadj2 + " " + monname + "\");\n");
      write_file(filenm, "      mon_ob->set_race(\"" + monname + "\");\n");
      write_file(filenm, "      mon_ob->set_wc(" +
                         "TUNE_MONSTER_WC(mon_ob->query_wc()));\n");
      write_file(filenm, "      mon_ob->set_ac(" +
                         "TUNE_MONSTER_AC(mon_ob->query_ac()));\n");
      write_file(filenm, "      mon_ob->set_hp(" +
                         "TUNE_MONSTER_HP(mon_ob->query_hp()));\n");
      write_file(filenm, "      mon_ob->set_whimpy(1);\n");
      write_file(filenm, "      mon_ob->set_random_pick(10);\n");
      write_file(filenm, "      /* mon_ob->set_al();            */\n");
      write_file(filenm, "      /* mon_ob->add_money();         */\n");
      write_file(filenm, "      /* mon_ob->set_gender();        */\n");
      write_file(filenm, "      /* mon_ob->set_aggressive();    */\n");
      write_file(filenm, "      /* mon_ob->can_move_at_reset(); */\n");
      write_file(filenm, "      /* mon_ob->load_chat();         */\n");
      write_file(filenm, "      /* mon_ob->load_a_chat();       */\n");
      write_file(filenm, "      /* mon_ob->set_chance();        */\n");
      write_file(filenm, "      /* mon_ob->set_spell_dam();     */\n");
      write_file(filenm, "      /* mon_ob->set_spell_mess1();   */\n");
      write_file(filenm, "      /* mon_ob->set_spell_mess2();   */\n");
      write_file(filenm, "      move_object(mon_ob, this_object());\n");
      write_file(filenm, "   }\n");
   }
   write_file(filenm, "}\n\n\n");


   /* Do we need to add any commands to the room? */
   if ((search_array(furnish_list, "kitchen"))     ||
       (search_array(furnish_list, "magicmirror"))) {
      write_file(filenm, "/* Add the furnish option commands to room */\n");
      write_file(filenm, "init() {\n");
      write_file(filenm, "   ::init();\n");
      if (search_array(furnish_list, "kitchen")) {
         write_file(filenm, "   add_action(\"furnish_cmd\", \"cook\");\n");
         write_file(filenm, "   add_action(\"furnish_cmd\", \"pour\");\n");
      }
      if (search_array(furnish_list, "magicmirror")) {
         write_file(filenm, "   add_action(\"furnish_cmd\", " +
                                          "\"concentrate\");\n");
      }
      write_file(filenm, "}\n\n\n");
      write_file(filenm, "/* Command function for furnish options */\n");
      write_file(filenm, "furnish_cmd(str) {\n");
      write_file(filenm, "   if (query_verb() == \"concentrate\") {\n");
      write_file(filenm, "      \"" + FURNISHOB + "\"->mirror_cmd(str);\n");
      write_file(filenm, "      return 1;\n");
      write_file(filenm, "   }\n");
      write_file(filenm, "   if ((query_verb() == \"cook\")  ||\n");
      write_file(filenm, "       (query_verb() == \"pour\")) {\n");
      write_file(filenm, "      return \"" + SUSTENANCE + "\"->order(str);\n");
      write_file(filenm, "   }\n");
      write_file(filenm, "}\n\n\n");
   }

   clock_list = 0;
   for (counter = 0; counter < sizeof(furnish_list); counter ++) {
      if (sscanf(furnish_list[counter], "clock:%s", discard) == 1) {
         clock_list = ({ furnish_list[counter] });
         break;
      }
   }


   /* What about examinable furnishings? */
   if ((search_array(furnish_list, "kitchen"))     ||
       (search_array(furnish_list, "magicmirror")) ||
       (clock_list)                                ||
       (search_array(furnish_list, "hourglass")))   {
      write_file(filenm, "/* id() - match things we want to examine */\n");
      write_file(filenm, "id(str) {\n");
      write_file(filenm, "   if (::id(str)) {\n");
      write_file(filenm, "      return;\n");
      write_file(filenm, "   }\n");

      if (search_array(furnish_list, "kitchen")) {
         write_file(filenm, "   if ((str == \"drinks\")         ||\n");
         write_file(filenm, "       (str == \"cabinet\")        ||\n");
         write_file(filenm, "       (str == \"drinks cabinet\") ||\n");
         write_file(filenm, "       (str == \"cupboard\")       ||\n");
         write_file(filenm, "       (str == \"ingredients\")    ||\n");
         write_file(filenm, "       (str == \"food\"))          {\n");
         write_file(filenm, "      return 1;\n");
         write_file(filenm, "   }\n");
      }
      if (search_array(furnish_list, "magicmirror")) {
         write_file(filenm, "   if ((str == \"mirror\")       ||\n");
         write_file(filenm, "       (str == \"magic mirror\")) {\n");
         write_file(filenm, "      return 1;\n");
         write_file(filenm, "   }\n");
      }
      if (clock_list) {
         write_file(filenm, "   if (str == \"clock\") {\n");
         write_file(filenm, "      return 1;\n");
         write_file(filenm, "   }\n");
      }
      /* This is the last one, so we can shortcut. */
      if (search_array(furnish_list, "hourglass")) {
         write_file(filenm, "   return (str == \"hourglass\");\n");
      }
      write_file(filenm, "}\n\n\n");
      write_file(filenm, "/* long() - description of examinable things */\n");
      write_file(filenm, "long(str) {\n");

      if (search_array(furnish_list, "kitchen")) {
         write_file(filenm, "   if ((str == \"drinks\")         ||\n");
         write_file(filenm, "       (str == \"cabinet\")        ||\n");
         write_file(filenm, "       (str == \"drinks cabinet\") ||\n");
         write_file(filenm, "       (str == \"cupboard\")       ||\n");
         write_file(filenm, "       (str == \"ingredients\")    ||\n");
         write_file(filenm, "       (str == \"food\"))          {\n");
         write_file(filenm, "      \"" + SUSTENANCE + "\"->list_items();\n");
         write_file(filenm, "      return;\n");
         write_file(filenm, "   }\n");
      }
      if (search_array(furnish_list, "magicmirror")) {
         write_file(filenm, "   if ((str == \"mirror\")       ||\n");
         write_file(filenm, "       (str == \"magic mirror\")) {\n");
         write_file(filenm, "      \"" + FURNISHOB + "\"->long_mirror();\n");
         write_file(filenm, "      return 1;\n");
         write_file(filenm, "   }\n");
      }
      if (clock_list) {
         write_file(filenm, "   if (str == \"clock\") {\n");
         write_file(filenm, "      \"" + FURNISHOB + "\"->long_clock(" +
                            "({ \"" + clock_list[0] + "\" }) );\n");
         write_file(filenm, "      return 1;\n");
         write_file(filenm, "   }\n");
      }
      /* This is the last one, so we can shortcut. */
      if (search_array(furnish_list, "hourglass")) {
         write_file(filenm, "   if (str == \"hourglass\") {\n");
         write_file(filenm, "      \""+MASTERHOUSE+"\"->hourglass_state();\n");
         write_file(filenm, "      return 1;\n");
         write_file(filenm, "   }\n");
      }
      write_file(filenm, "   return ::long(str);\n");
      write_file(filenm, "}\n\n\n");
   }

   return 1;
}



