
/* house_defines.h      Metaprat       16th April   */

/* Does the member_array() exist?  If it does, then set the define  */
/* MEMBER_ARRAY_EXISTS to true.  The definition is in the file      */
/* array_funcs.c.  It will improve the efficiency of the houses     */
/* significantly, since array searching is used very widely.  The   */
/* efun should be present in driver versions 3.0.27 and later.      */


/* Various definitions used in the houses */

#define WIZLEVEL     20

/* How much junk goes into /log/LOGFILE */
#define DEBUGLEVEL   1

/* Various files used in the houses */
#define METACASTLE   "/players/metaprat/castle"
#define HOUSES_DIR   "players/metaprat/houses/"
#define ROOMS        ("/" + HOUSES_DIR + "house_room")
#define INVITATIONS  ("/" + HOUSES_DIR + "house_invite")
#define MAPS         ("/" + HOUSES_DIR + "house_map")
#define ESTATEAGENT  ("/" + HOUSES_DIR + "estate_agent")
#define HOUSEKEEPER  ("/" + HOUSES_DIR + "house_keeper")
#define HOUSEHELP    ("/" + HOUSES_DIR + "house_help")
#define MONEY        ("/" + HOUSES_DIR + "house_money")
#define SUSTENANCE   ("/" + HOUSES_DIR + "house_sustenance")
#define HOUSEDRINK   ("/" + HOUSES_DIR + "house_drink")
#define INVITATIONS  ("/" + HOUSES_DIR + "house_invite")
#define MASTERHOUSE  ("/" + HOUSES_DIR + "master_house")
#define HOUSEKEY     ("/" + HOUSES_DIR + "house_key")
#define DOORSTATES   ("/" + HOUSES_DIR + "house_exit")
#define FURNISHOB    ("/" + HOUSES_DIR + "house_furnish_ob")
#define PORTAL       ("/" + HOUSES_DIR + "house_portal")
#define VOTING       ("/" + HOUSES_DIR + "house_voting")
#define HOUSE        ("/" + HOUSES_DIR + "house")

/* Toolkits, all inherited from TOOLKIT, and cloned at will. */

#define TOOLKIT      ("/" + HOUSES_DIR + "house_toolkit_obj")
#define BUILDER      ("/" + HOUSES_DIR + "house_builder")
#define CODEGEN      ("/" + HOUSES_DIR + "house_code_gen")
#define EDITDESC     ("/" + HOUSES_DIR + "house_desc_editor")
#define EDITITEM     ("/" + HOUSES_DIR + "house_item_editor")
#define EDITFURN     ("/" + HOUSES_DIR + "house_furnish_edit")
#define DESTROYER    ("/" + HOUSES_DIR + "house_destroyer")
#define ROOMSTATS    ("/" + HOUSES_DIR + "house_stats")

/* Filenames of datafiles.  Must NOT start with a leading "/"  */
/* Houses data files are HOUSES_DIR + name/NEW_OWNER, and  are */
/* saved/restored in save_stats(), restore_stats(name) in the  */
/* house.c & house_map.c objects.                              */
#define MASTER_DATA  (HOUSES_DIR + "master_house")
#define VOTING_DATA  (HOUSES_DIR + "house_voting")

/* Code fragments */
#define GETOWNER     main_hall->query_owner()
#define CHKSIZE(ITEM,LOW,HIGH) ( (ITEM >= LOW) && (ITEM <= HIGH) )
#define MYNAME       this_player()->query_real_name()
#define MYCNAME      this_player()->query_name()
#define IAMWIZ       (this_player()->query_level() >= WIZLEVEL)
#define IAMOWNER     (main_hall && (MYNAME == GETOWNER))

/* Logging.  By changing log_file to write_file, can log to your own */
/* directory   (change LOGFILE to include the full path).            */
/* (Note: This is unused and untested)                               */
#define LOGFILE      "metaprat.house"
#define LOGCMD(STR)  log_file(LOGFILE, STR);
#define LOGTHIS(LEV,STR) if (lev <= DEBUGLEVEL) { LOGCMD(STR); }

/* Standard locations within the mudlib */
/* used in special & wizard exits, HOUSEKEY, INVITATIONS */
#define STORAGE      "/room/storage"
#define CHURCH       "/room/church"
#define GUILD        "/room/adv_guild"
#define SHOP         "/room/shop"
#define POST         "/room/post"
#define PUB          "/room/pub2"
#define WORKROOM     ("/players/" + MYNAME + "/workroom")
#define CASTLE       ("/players/" + MYNAME + "/castle")
#define WIZBOARD     "/room/adv_inner"


/* Timeout period for maps & invites to turn to dust */
#define TIMEOUT      240

/* The number of rooms. Don't make it too high.. 30 is the ABSOLUTE limit */
#define MAXNUMROOMS  20       /* used in several places */

/* Maximum size of a description. Should be < max str size & max write size */
#define MAXDESCLEN   1200     /* used in DESCEDIT */
#define MAXDESCLINES 20
#define MAXITEMNUM   10       /* used in ITEMEDIT */
#define MAXITEMLEN   80

/* Charges for various building jobs etc.  MONEY file is important. */
#define HOUSECOST    5000
#define ROOMCOST     2000
#define KEEPERCOST   1000
#define FURNISHCOST  800
#define DECORCOST    200
#define DOORCOST     400
#define ITEMCOST     50
#define INVITECOST   50
#define INVITEPERMCOST 1000
#define MAPCOST      10

/* Door types, and state.  Main file concerned with door_state is */
/* HOUSEEXIT, which has a state transition table.                 */
#define DOOR_NONE    0
#define DOOR_NORM    1    /* normal exit */
#define DOOR_L_LOCK  10   /* lockable door, 3 state */
#define DOOR_L_UNLK  11
#define DOOR_L_OPEN  12
#define DOOR_O_CLOS  20   /* openable door, 2 state */
#define DOOR_O_OPEN  21
#define DOOR_PORTAL  30   /* owner only exit */
#define DOOR_ONEWAY  40   /* non-cardinal exits */
#define DOOR_HIDDEN  50   /* not listed in obvious dirs, or mapped */



/* Furnishings.  Set these to enable / disable each type of furnishing. */
/* Remember that the owner actually has to set the furnish option in    */
/* the room, as well as these defines being set.                        */
/* Recommend that clock, public, drink always be set, and the others    */
/* according to taste (although I think they're good ;)                 */
#define ALLOW_MIRROR     1  /* Allows players to see other players locations */
#define ALLOW_HOURGLASS  1  /* Gives a clue to the next reset time     */
#define ALLOW_CLOCK      1  /* Tells player real time                  */
#define ALLOW_PUBLIC     1  /* Anyone can enter house, if set by owner */
#define ALLOW_MONSTER    1  /* 1 monster per room can be created       */
#define ALLOW_FOOD       1  /* See the following defines for tuning    */
#define ALLOW_DRINK      1  /* food & drink. ('kitchen' option).       */

/*      Tuning definitions for the house sustenance code.  The ALLOW_     */
/* definitions control whether the item is allowed to sell drinks or food */
/* at all.  If it is, then the _ALLOW_COFFEE and _ALLOW_HEAL control      */
/* whether sobering drinks, and very high heal drinks are allowed to be   */
/* sold.  Note that they are very expensive in any case.                  */

/*      To fine tune the effectiveness of the food and drinks, set the    */
/* following: _STR_TUNE, controls how alcoholic or how filling the item   */
/* is.  The higher it is, the less they will be able to drink or eat.     */

/*      _COST_TUNE is now expensive it is.  _HEAL_TUNE is how much        */
/* healing the item does when it is first ingested, in addition to the    */
/* normal increase during the drunken period.  Note that food only uses   */
/* the strength to control this, and _HEAL_ is ignored.                   */

/*      Try to stick with the filenames mentioned, since other            */
/* implementations of food & drink may use different calls to set them    */
/* up.  If you use /obj/alco_drink, _allow_coffee and _allow_heal MUST be */
/* false, since it isn't flexible enough to be able to deal with them.    */

#define DRINK_FILE              HOUSEDRINK
#define DRINK_STR_TUNE(ARG)     (ARG * 100 / 100)
#define DRINK_COST_FORMULA(ARG) (4*ARG + ((ARG*ARG)/10))
#define DRINK_COST_TUNE(ARG)    (ARG * 100 / 100)
#define DRINK_HEAL_TUNE(ARG)    (ARG * 100 / 100)
#define DRINK_ALLOW_COFFEE      1
#define DRINK_ALLOW_HEAL        1

#define FOOD_FILE               "/obj/food"
#define FOOD_STR_TUNE(ARG)      (ARG * 100 / 100)
#define FOOD_COST_FORMULA(ARG)  (4*ARG + ((ARG*ARG)/10))
#define FOOD_COST_TUNE(ARG)     (ARG * 100 / 100)


/* This definition controls the amount of revenue a player gets when */
/* a visitor spends money in his house.  Note that he/she gets NO    */
/* money if the visitor is a wizard, and also that this only works   */
/* for visitors.. people don't get money from spending money in      */
/* their own houses.  Vary this between 5% and 35%.  To see how much */
/* money is given out as owner revenue, as compared to spending, do  */
/* 'show_revenue' once you have a house statistics object.           */
#define COMMISSION(spent) (spent * 20 / 100)


/* These defines are for the 'monster' furnishing option.  This allows */
/* players to create monsters in rooms.  There is a max of 1 monster   */
/* per room, and the entrance room may not have a monster set.  Note   */
/* also that there is NO money, & NO equipment on the monsters at all. */
/* The tuning applies to the default wc, ac, & hp set by the standard  */
/* /obj/monster.  Every mud has different rules as regards these, so   */
/* take care when changing these values.  The tuning is as follows:    */
/*   (((attribute + offset) * %increase / 100)                         */
#define MAXMONSTERLEVEL      19
#define TUNE_MONSTER_WC(WC)  ((WC + 1) * 110 / 100)
#define TUNE_MONSTER_AC(AC)  ((AC + 1) * 110 / 100)
#define TUNE_MONSTER_HP(HP)  ((HP + 0) * 130 / 100)

/* These defines are used for the voting object, which also has an optional */
/* quest (to be voted best house owner).                                    */
#define  HOUSE_VOTE_FILE (HOUSES_DIR + "house_vote")           /* data file */
#define  VOTEFORSELF    0                   /* owners vote for themselves?  */
#define  ALLOW_QUEST    0                   /* 1 if official quest          */
#define  QUESTNAME     "best_house"         /* don't change this            */
#define  EXPREWARD      2000                /* experience & money the       */
#define  MONEYREWARD    1000                /* winner gets when completed.  */
#define  REPEAT_WINNERS 0                   /* usually zero                 */
#define  MAXNUMVOTES    3                   /* should be 1, 3, or 5         */
#define  CANWIZVOTE     1                   /* if wizards can vote, = 1     */
#define  MINVOTINGLEV   3                   /* prevent 2nd-ary chars voting */
#define  FINISHTIME     (60 * 60 * 24 * 2)  /* make smaller for large muds  */
#define  FINISHVOTES    (11 * MAXNUMVOTES)  /* make larger for large muds   */



query_drop_castle() {
   /* prevent castles being dropped in this room */
   return 1;
}
