
 /* House_furnish_editor.c      Metaprat       23rd August 1992  */


inherit "/players/metaprat/houses/house_toolkit_obj";

#include "house_defines.h"


static string furnishings;
static string furnish_selected;
static string valid_furnishings;
static int    furnish_changed;

/* standard functions */


id(str) {
   return (str == "house furnish editor") || (str == "furnish editor") ||
          (str == "editor");
}


short() {
   return "house furnish editor";
}


long() {
   write("A house furnish editor.\n");
   write("It allows the owner of a house to modify the 'furnishings'\n");
   write("(capabilities of the room).  For example, if you give the\n");
   write("room a 'kitchen' capability, you can pour drinks, and cook\n");
   write("food in it  (help kitchen).\n");
   write("Commands available:\n");
   write("\tfurnish \t- Puts you into the furnish editor menu.\n");
   write("Once you are there, you are offered the following choices:\n");
   write("\tq       \t- quit furnish menu, loosing new furnish list\n");
   write("\tu       \t- update the room with the new furnish list\n");
   write("\ta <name>\t- add furnishing to the list\n");
   write("\td <name>\t- delete furnishing from the list\n");
   write("\tl       \t- list furnishings in the list\n");
   write("\th or ?  \t- help\n");
   write("The valid furnishings:\n");
   if (ALLOW_FOOD || ALLOW_DRINK) {
      write("\tkitchen\t- as described above.\n");
   }
   if (ALLOW_MIRROR) {
      write("\tmagicmirror\t- Locate other players within the game\n");
   }
   if (ALLOW_CLOCK) {
      write("\tclock\t- Find out what the time is ;)\n");
   }
   if (ALLOW_PUBLIC) {
      write("\tpublic\t- Allow anyone entry at any time\n");
   }
   if (ALLOW_HOURGLASS) {
      write("\thourglass\t- Gives hints as to when the next reset is\n");
   }
   if (ALLOW_MONSTER) {
      write("\tmonster\t- Create a monster in the room (Lev 1 to " +
            MAXMONSTERLEVEL + ")\n");
   }
   write("\tdarkness\t- Make the room dark\n");
}



reset(arg) {
   if (!arg) {
      if (ALLOW_FOOD || ALLOW_DRINK) {
         valid_furnishings = ({ "kitchen" });
      }
      if (ALLOW_MIRROR) {
         valid_furnishings = valid_furnishings + ({ "magicmirror" });
      }
      if (ALLOW_CLOCK) {
         valid_furnishings = valid_furnishings + ({ "clock" });
      }
      if (ALLOW_PUBLIC) {
         valid_furnishings = valid_furnishings + ({ "public" });
      }
      if (ALLOW_HOURGLASS) {
         valid_furnishings = valid_furnishings + ({ "hourglass" });
      }
      if (ALLOW_MONSTER) {
         valid_furnishings = valid_furnishings + ({ "monster" });
      }
      valid_furnishings = valid_furnishings + ({ "darkness" });
   }
   ::reset(arg);
}


toolkit_init() {
   add_action("furnish", "furnish");
}


write_furnish_menu() {
   write("Furnish menu:\n");
   write(" q        - quit furnish menu, & discard new furnish list\n");
   write(" u        - update the room with the new furnish list (cost: " +
         FURNISHCOST + ")\n");
   write(" a <name> - add a furnishing to the list\n");
   write(" d <name> - delete a furnishing from the list\n");
   write(" l        - list the new furnishings for the room\n");
   write(" h or ?   - help\n");
}


furnish(str) {
   object this_main_hall;

   this_main_hall = environment(this_player())->query_main_hall();
   if (!this_main_hall) {
      write("You are not in a house.\n");
      return 1;
   }
   if (this_main_hall != main_hall) {
      write("You are not in your own house.\n");
      return 1;
   }
   this_room_index = environment(this_player())->query_room_index();
   furnishings = main_hall->get_rooms_furnishings(this_room_index);
   if (!furnishings || !pointerp(furnishings)) {
      furnishings = ({ });
   }
   furnish_changed = 0;
   write_furnish_menu();
   write("List of valid furnishings:\n");
   write("[" + implode(valid_furnishings, "/") + "]\n");
   list_furnish();
   input_to("edit_furnish");
   return 1;
}



set_furnish_info() {
   object here;

   if (!furnish_changed) {
      write("None of the entries in the furnish list have been altered,\n");
      write("therefore there is nothing to update.\n");
      return 1;
   }
   MONEY->charge_money(FURNISHCOST, "furnish");
   write("You update " + main_hall->get_rooms_short(this_room_index) + ".\n");
   main_hall->set_rooms_furnishings(this_room_index, furnishings);
   main_hall->save_stats();
   if (environment(this_player())->confirm_house()) {
      /* if they've set the monster furnish, then make sure it is */
      /* created immediately, rather than waiting for a reset.    */
      /* Ditto for light.                                         */
      environment(this_player())->make_monster();
   }
   if (this_room_index == 0) {
      if (search_array(furnishings, "public")) {
         MASTERHOUSE->register_public_house(GETOWNER);
      }
      else {
         MASTERHOUSE->revoke_public_house(GETOWNER);
      }
   }
   this_room_index = -1;
   furnishings = ({ });
   return 1;
}


edit_furnish(str) {
   if (!MONEY->has_money(FURNISHCOST)) {
      return 1;
   }
   if (!str || (str == "")) {
      write_furnish_menu();
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   if (lower_case(str) == "q") {
      write("Leaving furnish menu, discarding new furnish list.\n");
      this_room_index = -1;
      furnishings = ({ });
      return 1;
   }
   if (lower_case(str) == "a") {
      write("Please enter the name of the furnishing to add : ");
      input_to("select_add_furnish");
      return 1;
   }
   if (sscanf(lower_case(str), "a %s", furnish_selected) == 1) {
      add_furnish();
      return 1;
   }
   if (lower_case(str) == "d") {
      write("Please enter the name of the furnishing to delete : ");
      input_to("select_delete_furnish");
      return 1;
   }
   if (sscanf(lower_case(str), "d %s", furnish_selected) == 1) {
      delete_furnish();
      return 1;
   }
   if (lower_case(str) == "l") {
      list_furnish();
      input_to("edit_furnish");
      return 1;
   }
   if (lower_case(str) == "u") {
      set_furnish_info();
      return 1;
   }
   if ((lower_case(str) == "h") || (str == "?")) {
      write("To add the 'kitchen' capability to a room:\n");
      write(" > furnish\n");
      write(" (the menu displayed here)\n");
      write(" Select: a kitchen                 (adding to list)\n");
      write(" Select: a monster:3:huge fat cook:cooks very nice food\n");
      write("                                   (adding to list)\n");
      write(" Select: l                         (list furnish in list)\n");
      write(" [1]  kitchen.\n");
      write(" [2]  monster:3:huge fat cook:cooks very nice food.\n");
      write(" Select: d 2                       (delete monster from list)\n");
      write(" Select l                          (list again, to check)\n");
      write(" [1]  kitchen.\n");
      write(" Select: u                         (update the room)\n");
      write(" You are charged " + FURNISHCOST + " coins.\n");
      write(" You update A large, spacious kitchen.\n");
      write(" > look at drinks cabinet\n");
      write(" (lots of drinks displayed)\n");
      write(" > look at ingredients\n");
      write(" (lots of food displayed)\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   write("Didn't understand furnish menu selection.\n");
   write_furnish_menu();
   write("Select: ");
   input_to("edit_furnish");
   return 1;
}



select_add_furnish(str) {
   if (str && (str != "")) {
      furnish_selected = str;
      add_furnish();
   }
   else {
      write("No furnish name provided.  Returning to furnish menu.\n");
      write("Select: ");
      input_to("edit_furnish");
   }
}



select_delete_furnish(str) {
   if (str && (str != "")) {
      furnish_selected = str;
      delete_furnish();
   }
   else {
      write("No furnish name provided.  Returning to furnish menu.\n");
      write("Select: ");
      input_to("edit_furnish");
   }
}



add_furnish() {
   string scanned_furnish;
   int    monlev;
   string monadj1;
   string monadj2;
   string monname;
   string add_or_sub;
   int    offset_hour;
   int    offset_min;
   int    counter;
   string discard;

   if (furnish_selected &&
       (sscanf(furnish_selected, "monster:%d:%s %s %s",
               monlev,
               monadj1,
               monadj2,
               monname) == 4)) {
      scanned_furnish = "monster";
      if (this_room_index == 0) {
         write("Sorry, no monsters may be added to the entrance room.\n");
         write("Select: ");
         input_to("edit_furnish");
         return 1;
      }
   }
   else if (furnish_selected &&
            (sscanf(furnish_selected, "clock:+%d.%d",
                    offset_hour,
                    offset_min) == 2)) {
      add_or_sub = "+";
      scanned_furnish = "clock";
   }
   else if (furnish_selected &&
            (sscanf(furnish_selected, "clock:-%d.%d",
                    offset_hour,
                    offset_min) == 2)) {
      add_or_sub = "-";
      scanned_furnish = "clock";
   }
   else if ((furnish_selected == "public") && (this_room_index != 0)) {
      write("The 'public' option can only be set in the entrance room.\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   else {
      scanned_furnish = furnish_selected;
   }
   if (!search_array(valid_furnishings, scanned_furnish)) {
      write("No such furnishing option exists.  You may select from the\n");
      write("following:\n");
      write("[" + implode(valid_furnishings, "/") + "]\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   for (counter = 0; counter < sizeof(furnishings); counter ++) {
      if (sscanf(furnishings[counter]+":junk",
                 scanned_furnish + ":%s", discard) == 1) {
         write("That has already been selected.\n");
         write("Select: ");
         input_to("edit_furnish");
         return 1;
      }
   }
   if (scanned_furnish == "monster") {
      if (!monlev) {
         write("The monster furnish option must be in the format:\n");
         write("  monster:<level>:<adjective1> <adj2> <name>:<description>\n");
         write("The ':<description>' part is optional.\n");
         write("eg)\n");
         write("  monster:18:huge red dragon       or perhaps\n");
         write("  monster:18:huge red dragon:the dragon has wings of fire\n");
         write("Select: ");
         input_to("edit_furnish");
         return 1;
      }
      else if ((monlev < 1) || (monlev > MAXMONSTERLEVEL)) {
         write("The monsters level must be between 1 and " +
               MAXMONSTERLEVEL + ".\n");
         write("Select: ");
         input_to("edit_furnish");
         return 1;
      }
   }
   else if (scanned_furnish == "clock") {
      if (!add_or_sub) {
         write("The clock furnish option must be in the format:\n");
         write("  clock:+01.15     clock:+00.00     clock:-00.45\n");
         write("where  + or -  signifies whether the offset will be added\n");
         write("or subtracted,  and the 00.00 is offset in hours and\n");
         write("minutes respectively.  Note that the +/- and hour.min\n");
         write("is NOT optional.\n");
         write("Select: ");
         input_to("edit_furnish");
         return 1;
      }
      else if ((offset_hour > 23) || (offset_hour < 0) ||
               (offset_min  > 59) || (offset_min  < 0)) {
         write("You can't offset more than 23 hours, nor 59 minutes.\n");
         write("Select: ");
         input_to("edit_furnish");
         return 1;
      }
   }
   furnish_changed = 1;
   furnishings = furnishings + ({ lower_case(furnish_selected) });
   write("Select: ");
   input_to("edit_furnish");
   return 1;
}



delete_furnish() {
   int tmp_size;
   int which_one;

   if (sizeof(furnishings) <= 0) {
      write("No furnishings in list.\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   tmp_size = sizeof(furnishings);
   furnishings = remove_array_item(furnishings, furnish_selected);
   if ((tmp_size == sizeof(furnishings))                &&
       (sscanf(furnish_selected, "%d", which_one) == 1) &&
       (which_one > 0)                                  &&
       (sizeof(furnishings) >= which_one))               {
      /* maybe they did    'd 1'  instead of   'd <thing>'   */
      furnish_selected = furnishings[which_one - 1];
      tmp_size = sizeof(furnishings);
      furnishings = remove_array_item(furnishings, furnish_selected);
   }
   if (tmp_size == sizeof(furnishings)) {
      write("Nothing removed from furnishing list.\n");
   }
   else {
      furnish_changed = 1;
      write("" + (tmp_size - sizeof(furnishings)) + " furnishing removed.\n");
   }
   write("Select: ");
   input_to("edit_furnish");
   return 1;
}



list_furnish() {
   int counter;

   if (sizeof(furnishings) <= 0) {
      write("No furnishings in list.\n");
      write("Select: ");
      input_to("edit_furnish");
      return 1;
   }
   for (counter = 0; counter < sizeof(furnishings); counter ++) {
      write("[" + (counter + 1)  + "]\t" + furnishings[counter] + ".\n");
   }
   write("Select: ");
   input_to("edit_furnish");
   return 1;
}



