
 /* House_furnish_ob.c   V2.s      Metaprat       920927   */

 /* To add a new furnishing, add the commands to use it to the init() in */
 /* house_room, and put a hook into this object from furnish_cmd().  If  */
 /* the object is examinable, change execu_rooms_id() and execu_rooms_   */
 /* _long() in house.c.  If the examine is complex, then put a hook in   */
 /* leading to this object.                                              */


#include "house_defines.h"
#include "array_funcs.c"


make_monster(furnishings) {
   string monster_options;
   object monster_ob;
   string monadj1;
   string monadj2;
   string monname;
   string monlong;
   int    monlev;
   int    counter;

   if ((!furnishings) || (!pointerp(furnishings))) {
      return;
   }

   /* Is there a monster option here?  Extract it. */
   for (counter = 0; counter < sizeof(furnishings); counter ++) {
      if (sscanf(furnishings[counter], "monster:%s", monster_options) == 1) {
         break;
      }
      monster_options = 0;  /* otherwise it'd be undefined */
   }
   if (!monster_options) {
      return;
   }

   /* Parse the monster furnish for the various parameters */
   if ((sscanf(monster_options, "%d:%s %s %s:%s",
               monlev, monadj1, monadj2, monname, monlong) != 5)  &&
       ((monlong = 0) ||    /* clear monlong */
        (sscanf(monster_options, "%d:%s %s %s",
                monlev, monadj1, monadj2, monname)         != 4))) {
      return;
   }

   /* Do sanity checks */
   if (monlev > MAXMONSTERLEVEL) {
      monlev = MAXMONSTERLEVEL;
   }
   if (monlev < 1) {
      monlev = 1;
   }

   /* Make the monster */
   monster_ob = clone_object("/obj/monster");
   monster_ob->set_name(monadj1 + " " + monadj2 + " " + monname);
   monster_ob->set_level(monlev);
   monster_ob->set_alias(monadj1 + " " + monname);
   monster_ob->set_alt_name(monadj2 + " " + monname);
   monster_ob->set_race(monname);
   if (monlong) {
      monster_ob->set_long(capitalize(monlong) + "\n");
   }
   monster_ob->set_random_pick(random(20));
   monster_ob->set_whimpy( random(3) != 0 );  /* 2/3 chance of being wimpy */
   monster_ob->set_wc( TUNE_MONSTER_WC(monster_ob->query_wc()) );
   monster_ob->set_ac( TUNE_MONSTER_AC(monster_ob->query_ac()) );
   monster_ob->set_hp( TUNE_MONSTER_HP(monster_ob->query_hp()) );
   return monster_ob;
}



mirror_cmd(str) {
   object target;
   object env_target;
   object fighting;
   object next_ob;
   string new_str;
   string tmp_text;
   string discard1;
   string discard2;
   string realm;
   int    anyone_else;

   /* Parse input string, and locate the target */
   if (!str || (str == "")) {
      write("On whose image do you wish to concentrate?\n");
      return 1;
   }
   target = find_player(str);
   if (!target) {
      if (sscanf(str, "on %s", new_str) == 1) {
         target = find_player(new_str);
      }
      else {
         target = find_living(str);
      }
   }
   if ((!target) || (!target->short())) {
      write("Hmm.. that image doesn't seem to be there.\n");
      return 1;
   }
   if (target->query_level() > WIZLEVEL) {
      tell_object(target, capitalize(MYNAME) + " tried to locate you.\n");
      write("For some reason, that image seems to be beyond your " +
            "powers of concentration.\n");
      return 1;
   }

   /* Found target */
   if (target->query_level() > (this_player()->query_level() + random(10))) {
      tell_object(target, "You feel that someone is watching you.\n");
   }
   write("As you concentrate, the mists in the mirror seem to clear.  "+
         "The image of\n" + capitalize(str) + " starts to twist and "+
         "turn inside the glass..\n" + capitalize(str));

   /* Describe the environment */
   env_target = environment(target);
   if ((!env_target) || (!env_target->short())) {
      write(" seems to be in Limbo.  How strange!\n");
   }
   else {
      tmp_text = " is in " + env_target->short() + ", ";
      if ((env_target->confirm_house()) &&
          (env_target->query_main_hall()->query_owner())) {
         if (strlen(tmp_text) > 45) {
            write(tmp_text + "\n");
            tmp_text = "";
         }
         tmp_text = tmp_text +
            "which is part of " +
            capitalize(env_target->query_main_hall()->query_owner()) +
            "'s wonderful house, ";
      }
      if (strlen(tmp_text) > 50) {
         write(tmp_text + "\n");
      }
      else {
         write(tmp_text);
      }
      tmp_text = "";
      if (sscanf("/" + file_name(env_target),
                 "%splayers/%s/%s", discard1, realm, discard2) == 3) {
         write("in the realm of " + capitalize(realm) + ".\n");
      }
      else {
         write("outside all realms.\n");
      }

      /* Say who target is fighting */
      fighting = target->query_attack();
      if (fighting                                       &&
          (fighting->short())                            &&
          (environment(fighting) == environment(target))) {
         write(target->query_name() + " is fighting " +
               fighting->query_name() + ".\n");
      }

      /* List other living things in the same room */
      anyone_else = 0;
      next_ob = first_inventory(env_target);
      while (next_ob) {
         if (living(next_ob)         &&
             (next_ob->short())      &&
             (next_ob->query_name()) &&
             (next_ob != target)     &&
             (next_ob != fighting))   {
            if (!anyone_else) {
               write("Also in the mirror, you see ");
               anyone_else = 1;
            }
            else {
               write(", ");
            }
            write(next_ob->query_name());
         }
         next_ob = next_inventory(next_ob);
      }
      if (anyone_else) {
         write(".\n");
      }
   }
   return 1;
}



long_mirror() {
   object people_list;
   string tmp_text;
   int    counter;

   write("This is a large mirror.  There is a wierd shifting mist " +
         "in front of the\nglass.  On the glass, there are a " +
         "number of partly formed images:\n");
   people_list = users();
   tmp_text = ({ });
   for (counter = 0; counter < sizeof(people_list); counter ++) {
      if ((!people_list[counter])                            ||
          (!people_list[counter]->short())                   ||
          (people_list[counter]->query_level() >= WIZLEVEL)) {
         continue;
      }
      tmp_text = tmp_text + ({ people_list[counter]->query_name() });
   }
   write(implode(tmp_text, ", ") + "\n");
   write("That is all you can see, unless you concentrate on one " +
         "of the images.\n");
   return;
}



long_clock(furnishings) {
   int    counter;
   int    offset_hour;
   int    offset_min;
   string add_or_sub;

   if ((!furnishings) || (!pointerp(furnishings))) {
      return;
   }

   for (counter = 0; counter < sizeof(furnishings); counter ++) {
      if (sscanf(furnishings[counter], "clock:+%d.%d",
                 offset_hour, offset_min)                   == 2) {
         add_or_sub = "+";
      }
      else if (sscanf(furnishings[counter], "clock:-%d.%d",
                 offset_hour, offset_min)                   == 2) {
         add_or_sub = "-";
      }
      else {
         continue;
      }
      write("The time is " + ctime(time()));
      if (offset_hour || offset_min) {
         write(" local, and the time in\n");
         write("the owner's timezone is ");
         if (add_or_sub == "-") {
            write(ctime(time() - ((offset_hour * 60 * 60) +
                                  (offset_min * 60))));
         }
         else {
            write(ctime(time() + ((offset_hour * 60 * 60) +
                                 (offset_min * 60))));
         }
         write(".\n");
      }
      else {
         write(" both locally and in your time zone.\n");
      }
      break;
   }
}

