

/* house_voting.c       Metaprat       920923    */


#include "house_defines.h"
#include "array_funcs.c"

string winners;            /* paired array of past winners:info  */
string candidates;         /* paired array of candidates:votes   */
string voters;             /* paired array of voters:votes       */
string last_contest_info;  /* string containing summary of last contest */
int    contest_started_at; /* start time of current contest      */
int    votes_so_far;       /* number of votes in current contest */


reset(arg) {
   if (arg) {
      return;
   }
   restore_object(VOTING_DATA);
   /* check all recorded variables, and initialise if unset, or illegal */
   if (!winners || !pointerp(winners)) {
      winners = ({ });
   }
   if (!candidates || !pointerp(candidates)) {
      candidates = ({ });
   }
   if (!voters || !pointerp(voters)) {
      voters = ({ });
   }
   if (!last_contest_info || !stringp(last_contest_info)) {
      last_contest_info = "There have been no previous contests.\n";
   }
   if (!contest_started_at || !intp(contest_started_at)) {
      contest_started_at = time();
   }
   if (!votes_so_far || !intp(votes_so_far)) {
      votes_so_far = 0;
   }
}


id(str) {
   return search_array( ({ "a voting booth", "voting booth", "booth",
                           "returning officer", "officer"}),
                        lower_case(str));
}


short() {
   return "A voting booth";
}


long(str) {
   if ((str == "officer") || (str == "returning officer")) {
      write("The Returning Officer is the official in charge of the\n");
      write("voting.  He is an old gent, heavy protected behind iron\n");
      write("bars, electric deathtraps, and land minds.  Hmm.  A lot\n");
      write("of protection for one old man.. perhaps the losers of the\n");
      write("contests get rather irate at times?\n");
      return;
   }
   write("A voting booth.\n");
   write("The booth allows you to vote for whatever house you think is\n");
   write("the best.  You are limited to the houses which are 'public',\n");
   write("and you also cannot vote for a person who has won previously.\n");
   write("Commands which are relevant:\n");
   write("   vote <person>\n");
   write("   showwinners\n");
   write("   showlastcontest\n");
   write("   declarefinished\n");
   write("   getreward\n");
   write("All of these have 'help' information.\n");
   if (IAMWIZ) {
      write("Wizards get the command 'voting_stats'.\n");
   }
   write("The Returning Officer is standing behind the booth.\n");
   write("Minimum time:  " + (FINISHTIME / (60 * 60)) + " hours.\t" +
         "Time so far:   " +((time() -contest_started_at) /3600) +" hours.\n");
   write("Minimum votes: " + FINISHVOTES  + ".\t\t" +
         "Votes so far:  " + votes_so_far + ".\n");
}



init() {
   add_action("help",            "help");
   add_action("vote",            "vote");
   add_action("showwinners",     "showwinners");
   add_action("showlastcontest", "showlastcontest");
   add_action("declarefinished", "declarefinished");
   add_action("getreward",       "getreward");
   if (IAMWIZ) {
      add_action("voting_stats", "voting_stats");
   }
}



help(str) {
   return HOUSEHELP->help(str);
}



vote(str) {
   int num_votes;

   if (!str || (str == "") || (!stringp(str))) {
      write("Usage:  vote <person>\n");
      return 1;
   }
   str = lower_case(str);
   if (!VOTEFORSELF && (MYNAME == str)) {
      write("I'm sure your house is wonderful, but you can't vote for\n");
      write("yourself.. vote for the next best house ;)\n");
      return 1;
   }
   if (IAMWIZ && !CANWIZVOTE) {
      write("Sorry, wizards cannot vote in this contest, since it may be\n");
      write("part of a quest.\n");
      return 1;
   }
   if ((this_player()->query_level()) < MINVOTINGLEV) {
      write("Sorry, voting is restricted to people of level " + MINVOTINGLEV +
            " and higher,\n");
      write("in order to prevent secondary characters being created.\n");
      say("Returning officer rips up the vote slip.\n");
      return 1;
   }
   say(MYCNAME + " goes to the voting booth, to register a vote.\n");
   if (!MASTERHOUSE->query_house_exists(str)) {
      write("No such house exists.\n");
      say("Returning officer rips up the vote slip.\n");
      return 1;
   }
   if (!MASTERHOUSE->query_public_house(str)) {
      write("That house is not public, and therefore you cannot vote for\n");
      write("its owner.\n");
      say("Returning officer rips up the vote slip.\n");
      return 1;
   }
   if ((!REPEAT_WINNERS) && search_paired_array(winners, str)) {
      write("That house is so good, that it has already won the contest,\n");
      write("and therefore cannot take part in future contests.\n");
      say("Returning officer rips up the vote slip.\n");
      return 1;
   }

   num_votes = search_paired_array(voters, MYNAME);
   if (num_votes >= MAXNUMVOTES) {
      write("You have already had all the votes you are entitled to.\n");
      say("Returning officer rips up the vote slip.\n");
      return 1;
   }
   else {
      write("You use one of your " + MAXNUMVOTES + ", and have " +
            (MAXNUMVOTES - num_votes - 1) + " left for this contest.\n");
   }

   /* Increment person's vote count */
   voters = remove_paired_array_item(voters, MYNAME);
   voters = ({ MYNAME, num_votes + 1 }) + voters;

   /* Increment number of votes, for this candidate */
   num_votes = search_paired_array(candidates, str);
   candidates = remove_paired_array_item(candidates, str);
   candidates = ({ str, num_votes + 1 }) + candidates;

   write("You register your vote for " + capitalize(str) + "'s house.\n");
   say(MYCNAME + " registers a vote for someone, at the voting booth.\n");

   votes_so_far ++;
   save_object(VOTING_DATA);

   if ((FINISHTIME <= (time() - contest_started_at)) &&
       (FINISHVOTES <= votes_so_far))                {
      say("Returning officer says something to " + MYCNAME + ".\n");
      write("The contest can be declared closed, since both enough time\n");
      write("has passed, and enough votes been cast, to make it valid.\n");
      write("The command to do this is 'declarefinished'\n");
   }
   return 1;
}


showwinners() {
   int counter;

   write("The list of previous contests, and their winners:\n");
   for (counter = 0; counter < sizeof(winners); counter += 2) {
      write(winners[counter] + " had " + winners[counter + 1][1] +
            " votes, " + winners[counter+1][2] + "% of the total,");
      write("  at " + ctime(winners[counter+1][3]) + ".\n");
   }
   return 1;
}


showlastcontest() {
   write(last_contest_info);
   return 1;
}


declarefinished() {
   int    counter;
   int    this_percent;
   int    this_votes;
   string this_winner;
   string tmp_name;
   string tmp_votes;
   string up_to;
   string item_swapped_flag;
   string count;

   say(MYCNAME + " asks the returning officer to finish the contest.\n");
   write("You ask the returning officer to finish the contest.\n");

   /* Can contest finish yet? */
   if (FINISHTIME > (time() - contest_started_at)) {
      write("Sorry, not enough time has elapsed for the contest to finish.\n");
      say("The returning officer shakes his head at " + MYCNAME + ".\n");
      return 1;
   }
   if (FINISHVOTES > votes_so_far) {
      write("Sorry, not enough people have voted for the contest to " +
            "finish.\n");
      say("The returning officer shakes his head at " + MYCNAME + ".\n");
      return 1;
   }

   /* Sort the candidates array, by number of votes received. */
   item_swapped_flag = 1;
   up_to = 4;
   while (item_swapped_flag) {
      item_swapped_flag = 0;
      for (count = 0; count <= sizeof(candidates) - up_to; count += 2) {
         if (candidates[count + 1] < candidates[count + 3]) {
            item_swapped_flag = 1;
            tmp_name  = candidates[count];
            tmp_votes = candidates[count + 1];
            candidates[count]     = candidates[count + 2];
            candidates[count + 1] = candidates[count + 3];
            candidates[count + 2] = tmp_name;
            candidates[count + 3] = tmp_votes;
         }
      }
      up_to += 2;
   }

   /* Check that contest hasn't hung */
   if (sizeof(candidates) > 2) {
      if (candidates[1] == candidates[3])  {
         write("Hmm.. the two front runners in the contest are perfectly\n");
         write("matched.. the contest must continue!\n");
         say("The returning officer shakes his head at " + MYCNAME + ".\n");
         return 1;
      }
   }

   this_winner = candidates[0];
   this_votes = candidates[1];
   this_percent = (this_votes * 100) / votes_so_far;
   winners = winners +
             ({ this_winner, ({ 0, this_votes, this_percent, time() }) });

   /* Write last_contest_info page */
   last_contest_info =
      "Contest finished at " + ctime(time()) + " with the glorious winner:\n" +
      capitalize(this_winner) +
      " gained " +this_percent +"% of the total votes (" +votes_so_far +")." +
      "  The other\ncandidates were (in order of votes):\n";
   for (counter = 2; counter < sizeof(candidates) - 1; counter += 2) {
      last_contest_info = last_contest_info + candidates[counter] + " with " +
         ((candidates[counter + 1] * 100) / votes_so_far) + "%\n";
   }

   /* Inform the people in room, or who happen to be on. */
   tell_room(environment(),
      "The Returning Officer, for the Best Houses contest, stands up and " +
      "announces:\n\"The winner of this contest is " + capitalize(this_winner)+
      "!\"\n");
   if (find_player(this_winner)) {
      tell_object(find_player(this_winner),
         "The Returning officer, for the Best Houses contest, tells you:\n" +
         "WELL DONE!!!!!  YOU HAVE WON THE BEST HOUSES CONTEST!!\n" +
         "Collect your reward from the voting booth in the storage room.\n");
   }

   /* reset variables required for the next voting round */
   votes_so_far       = 0;
   candidates         = ({ });
   voters             = ({ });
   contest_started_at = time();

   save_object(VOTING_DATA);
   return 1;
}


getreward() {
   /* winners is in the format:                                          */
   /*   ({   "name",   ({ collected_reward, votes, percentage, date }),  */
   /*        "next",   ({ etc })   });                                   */
   string winners_record;

   winners_record = search_paired_array(winners, MYNAME);
   if (!winners_record) {
      write("Officer pokes you in the chest.\n");
      write("Officer says: We have no record of you ever being a winner!\n");
      return 1;
   }
   write("Officer says: You had " + winners_record[1] + " votes, which was " +
         winners_record[2] + "% of the total votes cast.\n");
   write("Officer says: The contest was closed on " + ctime(winners_record[3])+
         ".\n");
   if (!winners_record[0]) {
      winners_record[0] = 1;
      winners = remove_paired_array_item(winners, MYNAME);
      winners = winners + ({ MYNAME, winners_record });
      save_object(VOTING_DATA);
      write("Officer congratulates you!\n");
      write("Officer says: Tell done, here's your reward.\n");
      this_player()->add_exp(EXPREWARD);
      this_player()->add_money(MONEYREWARD);
      if (EXPREWARD > 10) {
         write("You suddenly feel more experienced!\n");
      }
      if (MONEYREWARD > 10) {
         write("You suddenly feel richer!\n");
      }
   }
   else {
      write("Officer says: You have already collected your reward.\n");
      if (ALLOW_QUEST) {
         write("Officer says: However, in case you have lost your quest\n");
         write("Officer says: marker, I'll make sure its set.\n");
      }
   }
   if ((this_player()->set_quest(QUESTNAME)) && (ALLOW_QUEST)) {
      write("Officer says: Well done!  You have completed the best house " +
            "quest!\n");
   }
   return 1;
}


voting_stats() {
   int counter;
   if (!IAMWIZ) {
      return 0;
   }
   write("Statistics of voting object:\n");
   write("Data file:       " + VOTING_DATA       + ".o.\n");
   write("Quest enabled:   " + ALLOW_QUEST         + ".\n");
   write("Quest name:      " + QUESTNAME           + ".\n");
   write("Exp reward:      " + EXPREWARD           + ".\n");
   write("Money reward:    " + MONEYREWARD         + ".\n");
   write("Repeat winners:  " + REPEAT_WINNERS      + ".\n");
   write("Max votes each:  " + MAXNUMVOTES         + ".\n");
   write("Time to finish:  " + (FINISHTIME / 3600) + " hours.\n");
   write("votes to finish: " + FINISHVOTES         + ".\n");
   write("This contest started at " + ctime(contest_started_at) + ".\n");
   write("There have been " + votes_so_far + " vote(s) so far.\n");
   write("For information about winners, see 'showwinners'\n");
   write("For information on last contest, see 'showlastcontest'\n");
   write("Candidates:\n");
   for (counter = 0; counter < sizeof(candidates) - 1; counter += 2) {
      write("\t" + candidates[counter] + " has " + candidates[counter + 1] +
             " votes.\n");
   }
   write("Voters:\n");
   for (counter = 0; counter < sizeof(voters) - 1; counter += 2) {
      write("\t" + voters[counter] + " has made " + voters[counter + 1] +
             " votes.\n");
   }
   return 1;
}
