
 /* master_house.c      Metaprat       15th April 1992     */

 /* An object that stores an array of house information. */


#include "house_defines.h"
#include "array_funcs.c"

#define MASTER_HOUSE_FILE  (HOUSES_DIR + "master_house")

string        house_owners;    /* recorded */
string        public_houses;
string        wizlist_entries;
string        wizlist_cache;
string        revenue_entries;
string        revenue_cache;
int           reset_interval;

static string table_owners;    /* not recorded */
static object table_houses;
static int    table_times;
static int    reset_last;


reset(arg) {
   if (arg) {
      /* record reset times, for the hourglass */
      if ((reset_interval >= 500) && (reset_interval <= 50000)) {
         reset_interval = (reset_interval + time() - reset_last) / 2;
      }
      else {
         reset_interval = time() - reset_last;
      }
      reset_last = time();
      dec_and_update_wizlist();
      sort_wizlist();
      update_revenue();
      save_object(MASTER_DATA);
      return;
   }
   reset_last = time();
   house_owners    = ({ });
   public_houses   = ({ });
   wizlist_entries = ({ });
   wizlist_cache   = ({ });
   revenue_entries = ({ });
   revenue_cache   = ({ });
   table_owners    = ({ });
   table_houses    = ({ });
   table_times     = ({ });
   restore_object(MASTER_DATA);
}


query_house(owner_name) {
   int indx;

   for (indx = sizeof(table_owners) - 1; indx >= 0; indx --) {
      if ((table_owners[indx] == owner_name) && (table_houses[indx])) {
         table_houses[indx]->XXXXXXXX();  /* to take it out of swap-space */
         return table_houses[indx];
      }
   }
   return 0;
}


register_house(owner_name, house_ob) {
   table_owners = table_owners + ({ owner_name });
   table_houses = table_houses + ({ house_ob });
   table_times  = table_times +  ({ time() });
}


register_new_house(owner_name) {
   if (!search_array(house_owners, owner_name)) {
      house_owners = house_owners + ({ owner_name });
      save_object(MASTER_DATA);
   }
}


register_public_house(owner_name) {
   /* Add a house to the list of houses which are open to the  */
   /* public, but only if the house is known to exist already. */
   /* NOTE: just patching this won't be enough.  Whenever the house is */
   /* created, it checks the 'public' furnish option in the entrance   */
   /* room, and calls either this function or revoke_public_house.     */
   if (!search_array(house_owners, owner_name)) {
      return;
   }
   if (!search_array(public_houses, owner_name)) {
      public_houses = public_houses + ({ owner_name });
      save_object(MASTER_DATA);
   }
}


revoke_public_house(owner_name) {
   /* take a house off the list of houses which are open to the public */
   /* NOTE: just patching this won't be enough.  Whenever the house is */
   /* created, it checks the 'public' furnish option in the entrance   */
   /* room, and calls either this function or register_public_house.   */
   public_houses = remove_array_item(public_houses, owner_name);
   save_object(MASTER_DATA);
}



query_house_exists(owner_name) {
   return search_array(house_owners, owner_name);
}


query_public_house(owner_name) {
   return search_array(public_houses, owner_name);
}


master_stats() {
   show_active_houses(1);
   show_house_owners(1);
   show_public_houses(1);
   show_revenue(1);
   write("See also: room_stats, and agent_stats.\n");
}


show_active_houses(hide_see_also) {
   int    counter;

   write("List of currently active houses:  (" + sizeof(table_owners) +
         " in total)\n");
   for (counter = 0; counter < sizeof(table_owners); counter ++) {
      if (!table_houses[counter]) {
         write(table_owners[counter]            + "    \t"  +
               "**self_destructed**"            + "\t"  +
               ctime(table_times[counter])      + "\n");
      }
      else {
         write(table_owners[counter]                        + "    \t>>"  +
               extract(file_name(table_houses[counter]),16) + "\t"  +
               ctime(table_times[counter])                  + "\n");
      }
   }
   if (!hide_see_also) {
      write("See also: master_stats, show_house_owners, show_public_houses, "+
            "show_revenue,\nroom_stats, and agent_stats.\n");
   }
}


show_house_owners(hide_see_also) {
   string text;
   int    counter;
   int    line_cnt;

   text = "List of house owners:   (" + sizeof(house_owners) + " in total)\n";
   line_cnt = 0;
   for (counter = 0; counter < sizeof(house_owners); counter ++) {
      text = text + house_owners[counter];
      line_cnt ++;
      if (line_cnt > 4) {
         write(text + "\n");
         text = "";
         line_cnt = 0;
      }
      else if (strlen(house_owners[counter]) <= 7) {
         text = text + "       \t";
      }
      else {
         text = text + "\t";
      }
   }
   if (text != "") {
      write(text + "\n");
   }
   if (!hide_see_also) {
      write("See also: master_stats, show_active_houses, show_public_houses, "+
            "show_revenue,\nroom_stats, and agent_stats.\n");
   }
}


show_public_houses(hide_see_also) {
   string text;
   int    counter;
   int    line_cnt;

   text = "List of public houses:  (" + sizeof(public_houses) + " in total)\n";
   line_cnt = 0;
   for (counter = 0; counter < sizeof(public_houses); counter ++) {
      text = text + public_houses[counter];
      line_cnt ++;
      if (line_cnt > 4) {
         write(text + "\n");
         text = "";
         line_cnt = 0;
      }
      else if (strlen(public_houses[counter]) <= 7) {
         text = text + "       \t";
      }
      else {
         text = text + "\t";
      }
   }
   if (text != "") {
      write(text + "\n");
   }
   if (!hide_see_also) {
      write("See also: master_stats, show_active_houses, show_house_owners, "+
            "show_revenue,\nroom_stats, and agent_stats.\n");
   }
}



dec_and_update_wizlist() {
   int counter;
   int wizlist_amount;

   /* Reduce the size of entries in the wizlist by 0.5% every reset. */
   /* If the new score is zero, remove it from the array.            */
   for (counter = 0; counter < sizeof(wizlist_entries); counter += 2) {
      wizlist_entries[counter + 1] =
         (wizlist_entries[counter + 1] * 199) / 200;
      if (wizlist_entries[counter + 1] <= 0) {
         wizlist_entries = remove_paired_array_item(wizlist_entries,
                                                    wizlist_entries[counter]);
         counter -= 2;
      }
   }
   /* Now run over the wizlist cache, adding the entries to the old */
   /* wizlist_entries array one by one.                             */
   for (counter = 0; counter < sizeof(wizlist_cache); counter += 2) {
      wizlist_amount =
         search_paired_array(wizlist_entries, wizlist_cache[counter]);
      wizlist_entries =
         remove_paired_array_item(wizlist_entries, wizlist_cache[counter]);
      wizlist_entries = wizlist_entries +
         ({ wizlist_cache[counter], wizlist_amount+wizlist_cache[counter+1]});
   }
   wizlist_cache = ({ });
}



inc_wizlist(person) {
   /* Increment the wizlist cache entry for a player, by one.  Put the */
   /* new record in at the front of the array, since it will probably  */
   /* be required again very soon (high locality of reference).        */
   int wizlist_amount;

   if ((!this_player()) || (!person) || (person == MYNAME)) {
      return;
   }
   wizlist_amount = search_paired_array(wizlist_cache, person);
   wizlist_cache  = remove_paired_array_item(wizlist_cache, person);
   wizlist_cache  = ({ person, wizlist_amount + 1 }) + wizlist_cache;
}



static sort_wizlist(person) {
   /* Ok, so this is bubblesort, and it sucks, but remember that we're */
   /* sorting an almost-sorted list, and therefore only need a few     */
   /* iterations, using the bubblesort, whereas quicksort/heapsort     */
   /* would require many iterations, and therefore be slower.          */
   string tmp_name;
   int    tmp_amount;
   int    count;
   int    up_to;
   int    item_swapped_flag;

   item_swapped_flag = 1;
   up_to = 4;
   while (item_swapped_flag) {
      item_swapped_flag = 0;
      for (count = 0; count <= sizeof(wizlist_entries) - up_to; count += 2) {
         if (wizlist_entries[count + 1] < wizlist_entries[count + 3]) {
            item_swapped_flag = 1;
            tmp_name   = wizlist_entries[count];
            tmp_amount = wizlist_entries[count + 1];
            wizlist_entries[count]     = wizlist_entries[count + 2];
            wizlist_entries[count + 1] = wizlist_entries[count + 3];
            wizlist_entries[count + 2] = tmp_name;
            wizlist_entries[count + 3] = tmp_amount;
         }
      }
      up_to += 2;
   }
}


#define NUMPLAYERSSHOWN 15

show_wizlist(person) {
   int    counter;

   write("Player wizlist information:\n");
   for (counter = sizeof(wizlist_entries) - 2; counter >= 0; counter -= 2) {
      if ((counter <= (NUMPLAYERSSHOWN * 2))    ||
          (wizlist_entries[counter] == person)) {
         write("[" + (counter / 2 + 1) + "]\t" +
               wizlist_entries[counter + 1] +  "\t" +
               wizlist_entries[counter] + "\n");
      }
   }
}


update_revenue() {
   int counter;
   int revenue_amount;

   /* copy the revenue cache onto the revenue entries */
   for (counter = 0; counter < sizeof(revenue_cache); counter += 2) {
      revenue_amount =
         search_paired_array(revenue_entries, revenue_cache[counter]);
      revenue_entries =
         remove_paired_array_item(revenue_entries, revenue_cache[counter]);
      revenue_entries = revenue_entries +
         ({ revenue_cache[counter], revenue_amount+revenue_cache[counter+1]});
   }
   revenue_cache = ({ });
   /* Now lets see if we can give the revenue to the players */
   for (counter = 0; counter < sizeof(revenue_entries); counter += 2) {
      if ((find_player(revenue_entries[counter])) &&
          (query_ip_number(find_player(revenue_entries[counter])))) {
         tell_object(find_player(revenue_entries[counter]),
            "A robotic voice tells you: " +
            "Income from visitors to your house: " +
            revenue_entries[counter + 1] + " gold coins.\n" +
            "A robotic voice tells you: " +
            "It has been added to the money you are carrying.\n");
         find_player(revenue_entries[counter])->
            add_money(revenue_entries[counter + 1]);
         MONEY->register_given_money(revenue_entries[counter + 1]);
         revenue_entries =
            remove_paired_array_item(revenue_entries,revenue_entries[counter]);
         counter -= 2;
      }
   }
}


add_revenue(person, new_revenue) {
   /* Add a revenue value to the revenue cache entry for a player.  Put   */
   /* the updated record in at the front of the revenue cache list, since */
   /* there is high locality of reference.                                */
   int revenue_amount;

   if ((!this_player())||(new_revenue <= 0)||(!person)||(person == MYNAME)) {
      return;
   }
   if (!intp(new_revenue)) {
      return;
   }
   revenue_amount = search_paired_array(revenue_cache, person);
   if (!intp(revenue_amount)) {
      revenue_amount = 0;
   }
   revenue_cache  = remove_paired_array_item(revenue_cache, person);
   revenue_cache  = ({ person, revenue_amount + new_revenue }) + revenue_cache;
}



show_revenue(hide_see_also) {
   int counter;

   write("Revenue generated by houses, for the owners:\n");
   for (counter = 0; counter < sizeof(revenue_entries); counter += 2) {
      write(revenue_entries[counter+1]+"\t"+revenue_entries[counter] + "\n");
   }
   if (!hide_see_also) {
      write("See also: master_stats, show_active_houses, show_house_owners," +
            " show_public_houses,\nroom_stats, and agent_stats.\n");
   }
}




hourglass_state() {
   int time_percentage;

   if (!reset_interval) {
      reset_interval = 30 * 60;  /* 30 minutes is a reasonable first guess */
   }
   time_percentage = (time() - reset_last) * 100 / reset_interval;
   if (time_percentage > 99) {
      write("The sands of time have fallen.\n");
   }
   else if (time_percentage > 90) {
      write("The sands have almost completely fallen.\n");
   }
   else if (time_percentage > 80) {
      write("The sands are mostly fallen.\n");
   }
   else if (time_percentage > 70) {
      write("The sands are about two thirds fallen.\n");
   }
   else if (time_percentage > 60) {
      write("The sands are a little more than half fallen.\n");
   }
   else if (time_percentage > 50) {
      write("The sands are about half fallen.\n");
   }
   else if (time_percentage > 40) {
      write("The sands of time are almost half fallen.\n");
   }
   else if (time_percentage > 30) {
      write("The sands are about one third fallen.\n");
   }
   else if (time_percentage > 20) {
      write("The sands of time are mostly unfallen.\n");
   }
   else if (time_percentage > 10) {
      write("The sands of time have just started falling.\n");
   }
   else {
      write("The hourglass has been freshly turned.\n");
   }
}
